/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.geoviewer.LegendPalette;

/**
 * Class which groups treatment zone values with the same value
 * On value update will update the values of all the treatment zones it contains
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class TreatmentZoneGroup extends Number {

    public static final String PROP_VALUE = "value";
    public static final String PROP_COLOR = "color";
    PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    private Double value;
    private Color color;
    private Set<TreatmentZone> treatmentZones = new HashSet<>();
    private FieldTaskPalette palette;

    public TreatmentZoneGroup(Double value) {
        this.value = value;
    }

    public void addTreatmentZone(TreatmentZone treatmentZone) {
        if (java.util.Objects.equals(this.value, treatmentZone.getValue())) {
            this.treatmentZones.add(treatmentZone);
        }
    }
    
    public Set<TreatmentZone> getTreatmentZones(){
        return Collections.unmodifiableSet(treatmentZones);
    }

    public void removeTreatmentZone(TreatmentZone treatmentZone) {
        this.treatmentZones.remove(treatmentZone);
    }

    public Double getValue() {
        return value;
    }

    public void setValue(Double value) {
        Double old = this.value;
        this.value = value;
        for (TreatmentZone treatmentZone : treatmentZones) {
            treatmentZone.setValue(value);
        }
        propertyChangeSupport.firePropertyChange(PROP_VALUE, old, value);
        setColor();
    }

    public void setPalette(FieldTaskPalette palette) {
        this.palette = palette;
        palette.addPropertyChangeListener(LegendPalette.PROP_LEGEND_COLORS, new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                TreatmentZoneGroup.this.setColor();
            }
        });
        setColor();
    }

    private void setColor() {
        if (palette != null && value != null) {
            Color old = color;
            color = palette.getGradientColorForValue(value);
            propertyChangeSupport.firePropertyChange(PROP_COLOR, old, color);
        }
    }

    public Color getColor() {
        return color;
    }

    @Override
    public int intValue() {
        return value.intValue();
    }

    @Override
    public long longValue() {
        return value.longValue();
    }

    @Override
    public float floatValue() {
        return value.floatValue();
    }

    @Override
    public double doubleValue() {
        return value;
    }

    @Override
    public String toString() {
        return value != null ? value.toString() : "";
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    public void addPropertyChangeListener(String prop, PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(prop, listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(String prop, PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(prop, listener);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final TreatmentZoneGroup other = (TreatmentZoneGroup) obj;
        if (other.value == null) {
            return false;
        }
        if (!Objects.equals(this.value, other.value)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 41 * hash + Objects.hashCode(this.value);
        return hash;
    }

    public static class TreatmentZoneGroupComparator implements Comparator<TreatmentZoneGroup> {

        @Override
        public int compare(TreatmentZoneGroup o1, TreatmentZoneGroup o2) {
            if (o1 == null && o2 != null) {
                return -1;
            }
            if (o1 != null && o2 == null) {
                return 1;
            }
            if (o1 == o2) {
                return 0;
            }
            if (o1.getValue() == null && o2.getValue() != null) {
                return -1;
            }
            if (o1.getValue() != null && o2.getValue() == null) {
                return 1;
            }
            if (o1.getValue() == o2.getValue()) {
                return 0;
            }
            return (int) (o1.getValue() - o2.getValue());
        }
    }
}
